/*$Id: u_nodemap.cc,v 25.26 2005/06/03 01:24:39 al Exp $ -*- C++ -*-
 * Copyright (C) 2002 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *------------------------------------------------------------------
 * node name to number mapping -- for named nodes
 */
#include "e_node.h"
#include "u_nodemap.h"
/*--------------------------------------------------------------------------*/
static NODE ground_node("0",0);
/*--------------------------------------------------------------------------*/
NODE_MAP::NODE_MAP()
  : _count(0),
    _node_map(),
    _nodes()
{
  _node_map["0"] = &ground_node;
  _nodes.push_back(&ground_node);
}
/*--------------------------------------------------------------------------*/
NODE_MAP::NODE_MAP(const NODE_MAP&)
  : _count(0),
    _node_map(),
    _nodes()
{
  untested();
  _node_map["0"] = &ground_node;
  _nodes.push_back(&ground_node);
}
/*--------------------------------------------------------------------------*/
/* return a pointer to a node given its number
 * assert fails if out of range
 */
NODE* NODE_MAP::operator[](int number)const
{
  assert(number >= 0);
  assert(number <= _count);
  return _nodes[number];
}
/*--------------------------------------------------------------------------*/
/* return a pointer to a node given a string
 * returns NULL pointer if no match
 */
NODE* NODE_MAP::operator[](const std::string& s)
{
  return _node_map[s];
}
/*--------------------------------------------------------------------------*/
/* return a pointer to a node given a string
 * creates a new one if it isn't already there.
 */
NODE* NODE_MAP::new_node(const std::string& s)
{
  NODE* node = _node_map[s];
  if (!node) {
    node = new NODE(s, ++_count);
    _nodes.push_back(node);
    _node_map[s] = node;
  }
  assert(node);
  return node;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
